# grade.py                       
# -*- coding: utf-8 -*-

from .utils import *
from .api import *
from .loc import *
from .gr import *

import os
import textwrap
import sys

class Grader:

    __exitMessage = '\nLeaving grader and returning to GitRoom Manager...'

    def __init__(self, lgo):
        self.lgo = lgo
        self.grader = None

    def _listDir(self, pwdir):
        dir_list = []
        for f in os.listdir(pwdir):
            if not f.startswith('.'):
                dir_list.append(f)
        return dir_list

    def _chooseDorF(self, pwdir, dir_list):
        prompt = 'Please choose a file or directory'
        opts = ['-+- Top student repo directory -+-', '..']
        opts += dir_list
        opts += ['<< Exit grader >>']
        choice = pickOpt(prompt, opts)
        if choice == 0:
            return self.lgo.student_repo_dir
        elif choice == 1:
            return os.path.dirname(pwdir)
        elif choice == len(opts) - 1:
            return 0
        else:
            return os.path.join(pwdir, opts[choice])

    def _selectGrader(self):
        self.grader = input('Please give grader\'s name or initials: ').strip()

    def _commentFile(self):
        promptMessage('Add comments for end of file (hit return when finished)')
        com = textwrap.dedent(input('Comment:\n\n')).strip()
        com = textwrap.fill(com, width = 70)
        com = com.splitlines()
        comment_char = input('Please enter comment character (enter for none): ')
        comment = []
        for line in com:
            comment.append('{} '.format(comment_char) + line)
        comment = '\n'.join(comment)
        header = '\n\n{} COMMENTS from {}\n\n'.format(comment_char, self.grader)
        return header + comment
    
    def _gradeFile(self, file_to_grade):
        with open(file_to_grade, 'r') as f:
            print('-' * 34 + ' BEGIN FILE ' + '-' * 34)
            print(f.read())
            print('-' * 35 + ' END FILE ' + '-' * 35)

        comments = self._commentFile()

        with open(file_to_grade, 'a') as f:
            f.write(comments)

    def _choiceFlow(self, pwdir):
        choice = self._chooseDorF(pwdir, self._listDir(pwdir))
        if choice == 0:
            return self._quitGrader()
        elif os.path.isdir(choice):
            self._choiceFlow(choice)
        else:
            self._gradeFile(choice)

    def _quitGrader(self):
        print(self.__exitMessage)
        return 0

    def main(self):
        self._selectGrader()
        status = self._choiceFlow(self.lgo.student_repo_dir)

        if status != 0:
                   
            prompt = 'Would you like to grade another?'
            choice = pickOpt(prompt, ['Yes','No'])
            if choice == 0:
                self._choiceFlow(os.path.dirname(file_to_grade))
            else:
                self._quitGrader()
            




# loc.py
# -*- coding: utf-8 -*-

import os
import subprocess as sp

from .utils import *

class LocalGit:

    '''
    Class for local machine system management
    '''

    def __init__(self, master_repo = None, student_repo_dir = None):
        if master_repo:
            master_repo = os.path.expanduser(master_repo).rstrip('/')
        if student_repo_dir:
            student_repo_dir = os.path.expanduser(student_repo_dir).rstrip('/')
        self.master_repo = master_repo
        self.student_repo_dir = student_repo_dir

    def __str__(self):
        text = 'Local master repo: {}\n'.format(self.master_repo)
        text += 'Local student repo directory: {}'.format(self.student_repo_dir)
        return text

    def set_master_repo(self, master_repo = None):
        if master_repo:
            self.master_repo = master_repo.rstrip('/')
            return

        while True:
            prompt = 'Please give path to course master repository: '
            mrp = input(str(prompt)).strip()
            try:
                mrp = os.path.expanduser(mrp)
            
            except AttributeError:
                errorMessage('Please input a proper path.')
                continue

            self.master_repo = mrp.rstrip('/')
            return

    def set_student_repo_dir(self, student_repo_dir = None):
        if student_repo_dir:
            self.student_repo_dir = student_repo_dir.rstrip('/')
            return

        while True:
            prompt = 'Please give path to student repository directory: '
            srdp = input(str(prompt)).strip()
            try:
                srdp = os.path.expanduser(srdp)
                    
            except AttributeError:
                errorMessage('Please input a proper path.')
                continue

            self.student_repo_dir = srdp.rstrip('/')
            return          
                
    # ----------------------------------
    # Git processes
    # ----------------------------------

    def gitInit(self, repo):
        args = ["git", "-C", repo, "init"]
        bequiet = sp.run(args)
        bequiet = None

    def gitRemoteAdd(self, repo, remote, remote_name = 'origin'):
        args = ["git", "-C", repo, "remote", "add", remote_name, remote]
        bequiet = sp.run(args)
        bequiet = None
    
    def gitClone(self, repo, remote):
        args = ["git", "clone", remote, repo]
        bequiet = sp.run(args)
        bequiet = None

    def gitPull(self, repo):
        args = ["git", "-C", repo, "pull"]
        bequiet = sp.run(args)
        bequiet = None

    def gitPush(self, repo, remote_name = 'origin', local_branch = 'master'):
        args = ["git", "-C", repo, "push", "-u", remote_name, local_branch]
        bequiet = sp.run(args)
        bequiet = None

    def gitAdd(self, repo):
        args = ["git", "-C", repo, "add", "."]
        bequiet = sp.run(args)
        bequiet = None

    def gitCommit(self, repo, message):
        args = ["git", "-C", repo, "commit", "-m", message]
        bequiet = sp.run(args)
        bequiet = None
  
    # ----------------------------------
    # Local system file copy processes
    # ----------------------------------

    def createLocalRepo(self, student_repo):
        try:
            os.mkdir(os.path.join(self.student_repo_dir, student_repo))
        except FileExistsError:
            pass

    def masterToStudent(self, student_repo):
        args = ["rsync", "-r",
                "--exclude", "_*",  # protected directories
                "--exclude", ".*",  # ignore hidden dot files
                "--exclude", ".*/", # ignore hidden dot directories
                self.master_repo + "/",
                os.path.join(self.student_repo_dir, student_repo)]
        bequiet = sp.run(args)
        bequiet = None
        




# api.py
# -*- coding: utf-8 -*-

import getpass
import os
import pandas as pd
import requests

from .utils import *

gh_api = 'https://api.github.com/'

class Admin:

    '''
    Class for primary GitHub Classrom management functions
    '''
    
    def __init__(self, ghid = None, token_file = None, token = None):
        self.ghid = ghid
        self.token_file = token_file
        self.__token = token

    def __str__(self):
        t = None if self.__token is None else '[ Hidden ]'
        text = '\nGitHub ID:  {}\n'.format(self.ghid)
        text +=  'Token path: {}\n'.format(self.token_file)
        text +=  'Token:      {}\n'.format(t)
        return text
      

class Org:

    '''
    Class for GitHub organization
    '''

    def __init__(self, name = None, repos = None, members = None, teams = None):
        self.name = name
        self.repos = repos if isinstance(repos, dict) else {}
        self.members = members if isinstance(members, dict) else {}
        self.teams = teams if isinstance(teams, dict) else {}

    def __str__(self):
        text = '\nOrg Name: {0}\n'.format(self.name)
        if self.repos:
            text +=  '\nRepos:\n'
            for k,v in sorted(self.repos.items()):
                text += str(Repository(name = v.name, repo_id = v.repo_id))
        if self.members:
            text +=  '\nMembers:\n'
            for k,v in sorted(self.members.items()):
                text += str(Member(ghid = v.ghid, role = v.role))
        if self.teams:
            text +=  '\nTeams:\n'
            for k,v in sorted(self.teams.items()):
                text += str(Team(name = v.name, team_id = v.team_id,
                                 repos = v.repos, members = v.members))
        return text


class Repository:

    '''
    Class for repository information
    '''

    def __init__(self, name = None, repo_id = None):
        self.name = name
        self.repo_id = repo_id

    def __str__(self):
        return '\nRepo Name: {}; Repo ID: {}\n'.format(self.name, self.repo_id)


class Member:

    '''
    Class for GitHub organization member
    '''

    def __init__(self, ghid = None, role = None):
        self.ghid = ghid
        self.role = role

    def __str__(self):
        if self.role:
            return '{} [{}]\n'.format(self.ghid, self.role)
        else:
            return '{}\n'.format(self.ghid)


class Team:

    '''
    Class for GitHub organization team
    '''

    def __init__(self, team_id = None, name = None, repos = None, members = None):
        self.team_id = team_id
        self.name = name
        self.repos = repos if isinstance(repos, dict) else {}
        self.members = members if isinstance(members, dict) else {}

    def __str__(self):
        text = '\nTeam Name: {}; Team ID: {}\n'.format(self.name, self.team_id)
        for k,v in sorted(self.repos.items()):
            text += str(Repository(name = v.name, repo_id = v.repo_id))
        for k,v in sorted(self.members.items()):
            text += str(Member(ghid = v.ghid, role = v.role))
        return text


class Roster:
            
    '''
    Class for course roster information
    '''

    def __init__(self, path = None):
        self.path = path
        self.students = {}

    def __str__(self):
        text =  '\nPath:   {0}\n'.format(self.path)
        text +=   'Students: '
        for k,v in sorted(self.students.items()):
            text += str(Student(first_name = v.first_name,
                                last_name = v.last_name,
                                ghid = v.ghid))
        return text


class Student:

    '''
    Class for single student information
    '''

    def __init__(self, first_name = None, last_name = None, ghid = None):
        self.first_name = first_name
        self.last_name = last_name
        self.ghid = ghid

    def __str__(self):
        return '{} {} [{}]; '.format(self.first_name, self.last_name, self.ghid)


class RemoteGit:

    '''
    Remote git management operations
    '''
    
    def __init__(self):
        self.admin = None
        self.org = None
        self.roster = None

    def __str__(self):
        text = '\nAdmin:\n {}\n'.format(self.admin)
        text += '\nOrganization:\n {}\n'.format(self.org)
        text += '\nRoster:\n {}\n'.format(self.roster)
        return text

    # ------------------------
    # Admin functions
    # ------------------------

    def setLogin(self, ghid = None, **kwargs):
        if not ghid:
            return input('Please enter your GitHub id: ').strip()
        else:
            return ghid
                          
    def setToken(self, tokenfile = None, **kwargs):
        if tokenfile is not None:
            tfp = os.path.expanduser(tokenfile) 
            with open(tfp) as f:
                return tfp, f.read().strip()
        while True:
            tfp = input('Please enter path to token file: ').strip()
            tfp = os.path.expanduser(tfp)
            if os.path.isfile(tfp):
                with open(tfp) as f:
                    return tfp, f.read().strip()
            else:
                errorMessage('Not a proper file!')
                continue

    def setAPICreds(self, **kwargs):
        admin = Admin()
        admin.ghid = self.setLogin(**kwargs)
        admin.token_file, admin._Admin__token = self.setToken(**kwargs)
        self.admin = admin

    def setOrg(self, **kwargs):
        org = Org(**kwargs)
        if not org.name:
            mess = 'Please enter your organization name: '
            org.name = input(mess)
        self.org = org

    # ------------------------
    # Roster functions
    # ------------------------

    def readRosterCSV(self, rosterpath):
        rpath = os.path.expanduser(rosterpath)
        df = pd.read_csv(rpath)
        return df, rpath

    def buildRoster(self, rosterfile = None, **kwargs):

        rost = Roster(**kwargs)
        if rosterfile:
            roster, rpath = self.readRosterCSV(rosterfile)
        else:
            while True:
                mess = 'Please enter path to roster CSV file: '
                rosterfile = os.path.expanduser(input(mess).strip())
                if os.path.isfile(rosterfile):
                    roster, rpath = self.readRosterCSV(rosterfile)
                    break
                else:
                    errorMessage('Not a proper file!')
                    continue         

        rost.path = rpath

        # allow for different column names in roster file
        icols = list(roster.columns.values)
        ocols = {'first_name': 'first name',
                 'last_name': 'last name',
                 'ghid': 'GitHub ID'}
        miss = list(set(['first_name','last_name','ghid']) - set(icols))
        corr = {}
        for k,v in ocols.items():
            if k in miss:
                mess = 'Which roster column is the student\'s ' + v + '?'
                choice = pickOpt(mess, icols)
                corr[k] = choice
            else:
                corr[k] = roster.columns.get_loc(k)
                      
        for index, row in roster.iterrows():
            student = Student(first_name = row[corr['first_name']],
                              last_name = row[corr['last_name']],
                              ghid = row[corr['ghid']])
            key = student.last_name + '_' + student.first_name
            rost.students[key] = student
        self.roster = rost

        # option to rename roster names to match GitRoom requirements
        if len(miss) > 0:
            p = 'Do you want rename you roster file columns with GitRoom names?'
            choice = pickOpt(p, ['Yes', 'No'])
            if choice == 0:
                old = [icols[corr['last_name']],
                       icols[corr['first_name']],
                       icols[corr['ghid']]]
                roster.rename(columns = {old[0]:'last_name',
                                         old[1]:'first_name',
                                         old[2]:'ghid'},
                              inplace = True)
                roster.to_csv(rpath, index = False)

    # ------------------------
    # Requests API
    # ------------------------

    def getGR(self, url, **kwargs):
        auth = (self.admin.ghid, self.admin._Admin__token)
        resp = requests.get(url, auth = auth, **kwargs)
        return resp.json()

    def postGR(self, url, **kwargs):
        auth = (self.admin.ghid, self.admin._Admin__token)
        resp = requests.post(url, auth = auth, **kwargs)
        return resp

    def putGR(self, url, **kwargs):
        auth = (self.admin.ghid, self.admin._Admin__token)
        resp = requests.put(url, auth = auth, **kwargs)
        return resp 
        
    # ------------------------
    # Org functions: get
    # ------------------------

    def getMembers(self):
        url = gh_api + 'orgs/' + self.org.name + '/members'
        resp = self.getGR(url, params = {'role':'admin'})
        ad_members = {}
        for r in resp:
            ghid = r['login']
            member = Member(ghid = ghid, role = 'admin')
            ad_members[ghid] = member
        resp = self.getGR(url, params = {'role':'member'})
        members = {}
        for r in resp:
            ghid = r['login']
            member = Member(ghid = ghid, role = 'member')
            members[ghid] = member
        members.update(ad_members)
        self.org.members = members

    def getTeams(self):
        url = gh_api + 'orgs/' + self.org.name + '/teams'
        resp = self.getGR(url)
        teams = {}
        for r in resp:
            team = Team(team_id = r['id'], name = r['name'])
            url = gh_api + 'teams/' + str(team.team_id) + '/members'
            resp = self.getGR(url)
            members = {}
            for r in resp:
                ghid = r['login']
                member = Member(ghid = ghid)
                members[ghid] = member
            team.members = members
            url = gh_api + 'teams/' + str(team.team_id) + '/repos'
            resp = self.getGR(url)
            repos = {}
            for r in resp:
                repo = Repository(name = r['name'], repo_id = r['id'])
                repos[r['name']] = repo
            team.repos = repos
            teams[team.name] = team
        self.org.teams = teams
            
    def getRepos(self):
        url = gh_api + 'orgs/' + self.org.name + '/repos'
        resp = self.getGR(url)
        repos = {}
        for r in resp:
            repo = Repository(name = r['name'], repo_id = r['id'])
            repos[r['name']] = repo
        self.org.repos = repos

    # ------------------------
    # Org functions: post/put
    # ------------------------

    def createRemoteRepo(self, repo_name, private = False):
        url = gh_api + 'orgs/' + self.org.name + '/repos'
        json = {'name': repo_name}
        if private == True:
            json['private'] = 'true'
        return self.postGR(url, json = json)

    def addMember(self, member, role = 'member'):
        mid = str(self.roster.students[member].ghid)
        url = gh_api + 'orgs/' + self.org.name + '/memberships/' + mid
        params = {'role': role}
        return self.putGR(url, params = params)
        
    def addAdmin(self, github_id):
        url = gh_api + 'orgs/' + self.org.name + '/memberships/' + github_id
        params = {'role': 'admin'}
        return self.putGR(url, params = params)
                                      
    def createTeam(self, team_name, permission = 'push'):
        url = gh_api + 'orgs/' + self.org.name + '/teams'
        json = {'name': team_name, 'permission': permission}
        return self.postGR(url, json = json)

    def addMemberToTeam(self, team_name, member, role = 'member'):
        mid = str(self.roster.students[member].ghid)
        tid = str(self.org.teams[team_name].team_id)
        url = gh_api + 'teams/' + tid + '/memberships/' + mid
        params = {'role': role}
        return self.putGR(url, params = params)
     
    def addRepoToTeam(self, team_name, repo_name):
        tid = str(self.org.teams[team_name].team_id)
        url = gh_api + 'teams/' + tid + '/repos/' + self.org.name
        url += '/' + repo_name
        params = {'permission': 'push'}
        return self.putGR(url, params = params)
    
        
        



# gr.py
# -*- coding: utf-8 -*-

from .utils import *
from .api import *
from .loc import *

import json
import os

class GR:

    '''
    Class for GitRoom initialization/reading functions
    '''

    def __init__(self, rgo = None, lgo = None):
        self.rgo = rgo
        self.lgo = lgo

    def __str__(self):
        text = '\nGitHub ID: {}\n'.format(self.rgo.admin.ghid)
        text += 'GitHub token file: {}\n'.format(self.rgo.admin.token_file)
        text += 'Organization name: {}\n'.format(self.rgo.org.name)
        text += 'Roster file: {}\n'.format(self.rgo.roster.path)
        text += 'Local master repo.: {}\n'.format(self.lgo.master_repo)
        text += 'Local student repo. directory: {}\n'.format(self.lgo.student_repo_dir)
        return text
    
    def _storeGitRoomInfo(self):
        
        while True:
            prompt = 'Please give directory for saving: '
            odir = os.path.expanduser(input(prompt).strip())
            if not os.path.isdir(odir):
                errorMessage('Please input a directory path.')
                continue
            
            info = {'github_login': self.rgo.admin.ghid,
                    'github_token_file': self.rgo.admin.token_file,
                    'organization_name': self.rgo.org.name,
                    'roster_file': self.rgo.roster.path,
                    'master_repo': self.lgo.master_repo,
                    'student_repo_dir': self.lgo.student_repo_dir}
            
            with open(odir + '/' + self.rgo.org.name + '_grm.json', 'w') as f:
                json.dump(info, f, indent = 4)
                
            break

    def _initGitRoom(self, github_login = None, token_file = None,
                     orgname = None, roster_file = None, master_repo = None,
                     student_repo_dir = None):

        # remote git inits
        self.rgo = RemoteGit()
        self.rgo.setAPICreds(ghid = github_login, tokenfile = token_file)
        self.rgo.setOrg(name = orgname)
        self.rgo.buildRoster(rosterfile = roster_file)
        
        # local git inits
        self.lgo = LocalGit()
        self.lgo.set_master_repo(master_repo = master_repo)
        self.lgo.set_student_repo_dir(student_repo_dir = student_repo_dir)

        # option to store inits if any were missing
        if (not github_login or not token_file or not orgname
            or not roster_file or not master_repo or not student_repo_dir):
            print('*' * 50)
            print('\nThis is what you have entered:\n')
            print(self)
            print('*' * 50)
            prompt = 'Is this correct?'
            choice = pickOpt(prompt, ['Yes','No'])
            if choice == 0:
                prompt = 'Do you want to store GitRoom information in JSON file?'
                choice = pickOpt(prompt, ['Yes','No'])
                if choice == 0:
                    self._storeGitRoomInfo()

            else:
                prompt = 'Would you like to try again?'
                choice = pickOpt(prompt, ['Yes','No'])
                if choice == 0:
                    self._initGitRoom(github_login = None, token_file = None,
                                      orgname = None, roster_file = None,
                                      master_repo = None, student_repo_dir = None)
                else:
                    progExit()

        self.rgo.getMembers()
        self.rgo.getTeams()
        self.rgo.getRepos()
                    
    def _readGitRoomInfo(self, init_file_path):
        
        with open(init_file_path, 'r') as f:
            info = json.load(f)
            
            req_keys = ['github_login', 'github_token_file',
                        'organization_name', 'roster_file',
                        'master_repo', 'student_repo_dir']

            for k in req_keys:
                try:
                    info[k]
                except KeyError:
                    info[k] = None               

            self._initGitRoom(github_login = info['github_login'],
                              token_file = info['github_token_file'],
                              orgname = info['organization_name'],
                              roster_file = info['roster_file'],
                              master_repo = info['master_repo'],
                              student_repo_dir = info['student_repo_dir'])

    def getGitRoomObjs(self):
        if self.rgo and self.lgo:
            return
        else:
            while True:
                prompt = 'How would you like to enter GitRoom information?'
                opts = ['Manually', 'From JSON file']
                choice = pickOpt(prompt, opts)
                if choice == 0:
                    self._initGitRoom()
                else:
                    prompt = 'Please give path to GitRoom JSON file: '
                    grjson = os.path.expanduser(input(prompt).strip())
                    if os.path.isfile(grjson):
                        self._readGitRoomInfo(grjson)
                        break
                    else:
                        errorMessage('Not a file!')
                        continue

    # ----------------------------------
    # GitRoom main menu choices
    # ----------------------------------

    def buildGR(self, from_scratch = False):

        __rb = 'https://github.com/'

        to_add = []

        if not from_scratch:
            # compare roster to remote
            current = []
            for k,v in self.rgo.org.members.items():
                current.append(v.ghid)
                
            for k,v in self.rgo.roster.students.items():
                if v.ghid not in current:
                    to_add.append(k)
        else:
            for k,v in self.rgo.roster.students.items():
                to_add.append(k)

        if len(to_add) == 0:
            print('All local students on remote')
            return
 
        m = 'Students to be added to {}'.format(self.rgo.org.name)
        promptMessage(m, char = '')
        for name in to_add:
            fn = self.rgo.roster.students[name].first_name
            ln = self.rgo.roster.students[name].last_name
            print('{} {}'.format(fn, ln))

        prompt = 'Should repos be private?'
        choice = pickOpt(prompt, ['Yes','No'])
        if choice == 0:
            priv_bool = True
        else:
            priv_bool = False

        # STEPS
        # 1. [Remote] Add student as GitRoom member
        # 2. [Remote] Init remote repo
        # 3. [Remote] Add team for student's repo       
        # 4. [Remote] Add student to team
        # 5. [Remote] Add repo to team
        # 6. [Local] Create student's local repo
        # 7. [Local] Copy files from master to student's repo
        # 8. [Local] Add, commit, push local student repo to remote
        
        for name in to_add:
            ln = self.rgo.roster.students[name].last_name
            gh = self.rgo.roster.students[name].ghid
            rn = 'student_{}'.format(ln.lower())
            rp = os.path.join(self.lgo.student_repo_dir, rn)

            # 1
            resp = self.rgo.addMember(member = name)

            if round(resp.status_code, -2) == 400:
                print(resp.json()['message'])
            elif resp.json()['state'] == 'active':
                print('{} is already an active member.'.format(name))
            elif resp.json()['state'] == 'pending':
                print('{} has a pending membership.'.format(name))

            # 2
            resp = self.rgo.createRemoteRepo(repo_name = rn, private = priv_bool)

            if resp.status_code == 422:
                text = '\nEither:\n\n'
                text += '(1) Remote already exists\n'
                text += '(2) Your organization plan doesn\'t allow for private repos \n'
                text += '    and you must change the setting to public \n'
                text += '    or upgrade your plan through GitHub.'
                errorMessage(text)        
            elif round(resp.status_code, -2) == 200:
                print('Successfully created remote {}'.format(rn))

            # 3
            resp = self.rgo.createTeam(team_name = rn)

            if resp.status_code == 422:
                print('Team {} already exists!\n'.format(rn))                      
            elif round(resp.status_code, -2) == 200:
                print('Successfully created team: {}'.format(rn))
                resp = resp.json()            
                team = Team(team_id = resp['id'], name = resp['name'])
                self.rgo.org.teams[team.name] = team

            # 4
            resp = self.rgo.addMemberToTeam(team_name = rn, member = name)

            if round(resp.status_code, -2) == 200:
                state = resp.json()['state']
                print('{}\'s membership on team {} is now {}.'.format(name,
                                                                      rn,
                                                                      state))
                mem = Member(ghid = gh)
                members = {}
                members[gh] = mem

            # 5
            resp = self.rgo.addRepoToTeam(team_name = rn, repo_name = rn)

            if round(resp.status_code, -2) == 200:
                print('{} now has access to repo {}'.format(rn, rn))

            # 6
            self.lgo.createLocalRepo(student_repo = rn)

            # 7
            self.lgo.masterToStudent(student_repo = rn)

            # 8
            remote = '{}{}/{}.git'.format(__rb, self.rgo.org.name, rn)
            self.lgo.gitInit(repo = rp)
            self.lgo.gitRemoteAdd(repo = rp, remote = remote)
            self.lgo.gitAdd(repo = rp)
            self.lgo.gitCommit(repo = rp, message = 'Init course repo')
            self.lgo.gitPush(repo = rp)

    def addGRAdmin(self):

        prompt = 'Please give new administrator\'s GitHub id: '
        ghid = input(prompt).strip()
        resp = self.rgo.addAdmin(github_id = ghid)

        if round(resp.status_code, -2) == 400:
            errorMessage(resp.json()['message'])
        elif round(resp.status_code, -2) == 200:
            print('Successfully added {} to {} as admin.'.format(ghid,
                                                                 self.org.name))

    def cloneGR(self):
        self.rgo.getRepos()
        while True:
            prompt = 'Please give path to local clone directory: '
            cdir = input(prompt).strip()
            try:
                cdir = os.path.expanduser(cdir)
                
            except AttributeError:
                errorMessage('Please give a proper path.')
                continue
            
            cdir = cdir.rstrip('/')
            break
        
        for k,v in self.rgo.org.repos.items():
            repo = os.path.join(cdir, k)
            remote = '{}{}/{}.git'.format(__rb, self.rgo.org.name, k)
            self.lgo.gitClone(repo = repo, remote = remote)

    def updateGR(self):
        self.rgo.getRepos()
        while True:
            prompt = 'Please enter commit message: '
            comment = input(prompt).strip()
            prompt = 'Is this okay or do you want to try again?\n\n'
            prompt += 'COMMENT: {}'.format(comment)
            choice = pickOpt(prompt, ['Good', 'Try again'])
            if choice == 0:
                break

        for k,v in self.rgo.org.repos.items():
            repo = k
            repo_path = os.path.join(self.lgo.student_repo_dir, repo)
            if not os.path.isdir(repo_path):
                continue
            self.lgo.gitPull(repo = repo_path)
            self.lgo.masterToStudent(student_repo = repo)
            self.lgo.gitAdd(repo = repo_path)
            self.lgo.gitCommit(repo = repo_path, message = comment)
            self.lgo.gitPush(repo = repo_path)  

    def pullGR(self):
        self.rgo.getRepos()

        prompt = 'Please select the student repository you wish to pull'
        opts = ['-- All student repositories --']
        for k,v in self.rgo.org.repos.items():
            repo = os.path.join(self.lgo.student_repo_dir, k)
            if not os.path.isdir(repo):
                continue
            opts.append(k)

        choice = pickOpt(prompt, opts)
        if choice == 0:
            for k,v in self.rgo.org.repos.items():
                repo_path = os.path.join(self.lgo.student_repo_dir, k)
                if not os.path.isdir(repo_path):
                    continue
                self.lgo.gitPull(repo = repo_path)
        else:
            repo_path = os.path.join(self.lgo.student_repo_dir, opts[choice])
            self.lgo.gitPull(repo = repo_path)

    def pushGR(self):
        self.rgo.getRepos()

        prompt = 'Please select the student repository you wish to push'
        opts = ['-- All student repositories --']
        for k,v in self.rgo.org.repos.items():
            repo_path = os.path.join(self.lgo.student_repo_dir, k)
            if not os.path.isdir(repo_path):
                continue
            opts.append(k)

        choice = pickOpt(prompt, opts)
            
        while True:
            comment = input('Please enter commit message: ').strip()
            prompt = 'Is this okay or do you want to try again?\n\n'
            prompt += 'COMMENT: {}'.format(comment)
            select = pickOpt(prompt, ['Good', 'Try again'])
            if select == 0:
                break
            
        if choice == 0:
            for k,v in self.rgo.org.repos.items():
                repo_path = os.path.join(self.lgo.student_repo_dir, k)
                if not os.path.isdir(repo_path):
                    continue
                self.lgo.gitPull(repo = repo_path)
                self.lgo.gitAdd(repo = repo_path)
                self.lgo.gitCommit(repo = repo_path, message = comment)
                self.lgo.gitPush(repo = repo_path)
        else:
            repo_path = os.path.join(self.lgo.student_repo_dir, opts[choice])
            self.lgo.gitPull(repo = repo)
            self.lgo.gitPull(repo = repo_path)
            self.lgo.gitAdd(repo = repo_path)
            self.lgo.gitCommit(repo = repo_path, message = comment)
            self.lgo.gitPush(repo = repo_path)



# utils.py
# -*- coding: utf-8 -*-

import sys

def printOpts(optlist):
    for l in optlist:
        print('(', optlist.index(l) + 1, ')', l)

def promptMessage(text, char = '-'):
    print('\n')
    print(char * len(text))
    print(text)
    print(char * len(text), end = '\n\n')

def errorMessage(text):
    print('\n')
    print('ERROR: ', end = '')
    print(text, end = '\n\n')

def pickOpt(prompt, optlist):

    em1 = 'Only digits are accepted. Please choose again.'
    em2 = 'Please choose again from among the options.'

    while True:
        promptMessage(prompt)
        printOpts(optlist)
        try:
            choice = int(input('\nCHOICE: '))

        except ValueError:
            errorMessage(em1)
            continue

        except EOFError:
            print('\n')
            raise SystemExit

        if choice > len(optlist) or choice < 1:
            errorMessage(em2)
            continue

        return choice - 1

def progExit():
    print('\n\n{}{}{}\n\n'.format('-' * 28,' Exit GitRoom Manager ', '-' * 28))
    sys.exit()



# __init__.py
# -*- coding: utf-8 -*-

from .__info__ import __version__, __au, __rp

from .gr import *
from .api import *
from .loc import *
from .img import *
from .grade import *
from .utils import *

def main():

    def __menu():

        prompt = 'What would you like to do?'
        opts = ['Initialize class',
                'Add students to class',
                'Add new administrator',
                'Clone repositories',
                'Update student remote repositories with local master',
                'Pull down from remote student repositories',
                'Push from local student repositories to remotes',
                'Grade student assignments',
                '<< Exit Program >>']
        choice = pickOpt(prompt, opts)

        # exit program
        if choice == len(opts) - 1:
            return

        # update GitRoom Objects if cycling through program
        gr.getGitRoomObjs()
                             
        # (1) Initialize class
        if choice == 0:

            gr.buildGR(from_scratch = True)

        # (2) Add students to class
        elif choice == 1:
            
            gr.buildGR()
        
        # (3) Add new administrator
        elif choice == 2:

            gr.addGRAdmin()
            
        # (4) Clone repositories
        elif choice == 3:

            gr.cloneGR()

        # (5) Update student remote repositories with local master
        elif choice == 4:

            gr.updateGR()
            
        # (6) Pull down from remote student repositories
        elif choice == 5:

            gr.pullGR()

        # (7) Push from local student repository to remote
        elif choice == 6:

            gr.pushGR()

        # (8) Grade student assignments
        else:
            grader = Grader(gr.lgo)
            grader.main()
            prompt = 'Do you want to push comments to student remotes?'
            choice = pickOpt(prompt, ['Yes','No'])
            if choice == 0:
                gr.pushGR()

        prompt = 'Do you have other tasks or wish to exit GitRoom Manager?'
        choice = pickOpt(prompt, ['I have another task', 'Exit'])

        if choice == 0:
            __menu()

    # ----------------------------------        
    # Run
    # ----------------------------------

    print(startup.format(__version__, __au, __rp))
    gr = GR()
    __menu()
    progExit()



__version__ = '0.1.6'
__gh = 'https://github.com/btskinner/'
__nm = 'grm'
__rp = '{}{}'.format(__gh, __nm)
__dl = '{}{}/archive/{}-v{}.tar.gz'.format(__gh, __nm, __nm, __version__)
__au = 'Benjamin Skinner'
__em = 'b.skinner@vanderbilt.edu'



# setup.py
# -*- coding: utf-8 -*-

from setuptools import setup
from distutils.util import convert_path

def readme():
    with open('README.rst') as f:
        return f.read()

info = {}
with open(convert_path('grm/__info__.py')) as f:
    exec(f.read(), info)

setup(name=info['__nm'],
      version=info['__version__'],
      description='GitHub classroom manager',
      long_description=readme(),
      classifiers=[
          'Development Status :: 3 - Alpha',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 3.5',
      ],
      keywords='github organization classroom manager',
      url=info['__rp'],
      download_url=info['__dl'],
      author=info['__au'],
      author_email=info['__em'],
      license='MIT',
      packages=['grm'],
      install_requires=[
          'pandas',
          'requests',
      ],
      scripts=['bin/gitroom'],
      zip_safe=False)
